@@ -50,11 +50,15 @@ struct _CcBackgroundChooserDialogPrivate
   GtkWidget *icon_view;
   GtkWidget *empty_pictures_box;
   GtkWidget *sw_content;
+  GtkWidget *pictures_button;
+  GtkWidget *colors_button;
 
   BgWallpapersSource *wallpapers_source;
   BgPicturesSource *pictures_source;
   BgColorsSource *colors_source;
 
+  GtkTreeRowReference *item_to_focus;
+
   GnomeDesktopThumbnailFactory *thumb_factory;
 
   GCancellable *copy_cancellable;
@@ -63,6 +67,7 @@ struct _CcBackgroundChooserDialogPrivate
 
   gulong row_inserted_id;
   gulong row_deleted_id;
+  gulong row_modified_id;
 };
 
 #define CC_CHOOSER_DIALOG_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), CC_TYPE_BACKGROUND_CHOOSER_DIALOG, CcBackgroundChooserDialogPrivate))
@@ -72,6 +77,17 @@ enum
   PROP_0,
 };
 
+enum
+{
+  URI_LIST,
+  COLOR
+};
+
+static const GtkTargetEntry color_targets[] =
+{
+  { "application/x-color", 0, COLOR }
+};
+
 G_DEFINE_TYPE (CcBackgroundChooserDialog, cc_background_chooser_dialog, GTK_TYPE_DIALOG)
 
 static void
@@ -118,6 +134,7 @@ cc_background_chooser_dialog_dispose (GObject *object)
       g_clear_object (&priv->copy_cancellable);
     }
 
+  g_clear_pointer (&chooser->priv->item_to_focus, gtk_tree_row_reference_free);
   g_clear_object (&priv->pictures_source);
   g_clear_object (&priv->colors_source);
   g_clear_object (&priv->wallpapers_source);
@@ -150,6 +167,38 @@ possibly_show_empty_pictures_box (GtkTreeModel              *model,
     }
 }
 
+static void
+on_source_modified_cb (GtkTreeModel *tree_model,
+                       GtkTreePath  *path,
+                       GtkTreeIter  *iter,
+                       gpointer      user_data)
+{
+  GtkTreePath *to_focus_path;
+  CcBackgroundChooserDialog *chooser = user_data;
+  CcBackgroundChooserDialogPrivate *priv = chooser->priv;
+
+  if (chooser->priv->item_to_focus == NULL)
+    return;
+
+  to_focus_path = gtk_tree_row_reference_get_path (chooser->priv->item_to_focus);
+
+  if (gtk_tree_path_compare (to_focus_path, path) != 0)
+    goto out;
+
+  /* Change source */
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (priv->pictures_button), TRUE);
+
+  /* And select the newly added item */
+  gtk_icon_view_select_path (GTK_ICON_VIEW (chooser->priv->icon_view), to_focus_path);
+  gtk_icon_view_scroll_to_path (GTK_ICON_VIEW (chooser->priv->icon_view),
+                                to_focus_path, TRUE, 1.0, 1.0);
+  g_clear_pointer (&chooser->priv->item_to_focus, gtk_tree_row_reference_free);
+
+out:
+  gtk_tree_path_free (to_focus_path);
+
+}
+
 static void
 on_source_added_cb (GtkTreeModel *model,
                     GtkTreePath  *path,
@@ -172,6 +221,9 @@ monitor_pictures_model (CcBackgroundChooserDialog *chooser)
 {
   GtkTreeModel *model;
 
+  if (chooser->priv->row_inserted_id != 0)
+    return;
+
   model = GTK_TREE_MODEL (bg_source_get_liststore (BG_SOURCE (chooser->priv->pictures_source)));
 
   chooser->priv->row_inserted_id = g_signal_connect (model, "row-inserted",
@@ -182,6 +234,10 @@ monitor_pictures_model (CcBackgroundChooserDialog *chooser)
                                                     G_CALLBACK (on_source_removed_cb),
                                                     chooser);
 
+  chooser->priv->row_modified_id = g_signal_connect (model, "row-changed",
+                                                     G_CALLBACK (on_source_modified_cb),
+                                                     chooser);
+
   possibly_show_empty_pictures_box (model, chooser);
 }
 
@@ -204,6 +260,12 @@ cancel_monitor_pictures_model (CcBackgroundChooserDialog *chooser)
       chooser->priv->row_deleted_id = 0;
     }
 
+  if (chooser->priv->row_modified_id > 0)
+    {
+      g_signal_handler_disconnect (model, chooser->priv->row_modified_id);
+      chooser->priv->row_modified_id = 0;
+    }
+
   ensure_iconview_shown (chooser);
 }
 
@@ -252,6 +314,98 @@ on_item_activated (GtkIconView               *icon_view,
   gtk_dialog_response (GTK_DIALOG (chooser), GTK_RESPONSE_OK);
 }
 
+static void
+add_custom_wallpaper (CcBackgroundChooserDialog *chooser,
+                      const char                *uri)
+{
+  g_clear_pointer (&chooser->priv->item_to_focus, gtk_tree_row_reference_free);
+
+  monitor_pictures_model (chooser);
+  bg_pictures_source_add (chooser->priv->pictures_source, uri, &chooser->priv->item_to_focus);
+  /* and wait for the item to get added */
+}
+
+static gboolean
+cc_background_panel_drag_color (CcBackgroundChooserDialog *chooser,
+                                GtkSelectionData          *data)
+{
+  gint length;
+  guint16 *dropped;
+  GdkRGBA rgba;
+  GtkTreeRowReference *row_ref;
+  GtkTreePath *to_focus_path;
+
+  length = gtk_selection_data_get_length (data);
+
+  if (length < 0)
+    return FALSE;
+
+  if (length != 8)
+    {
+      g_warning ("%s: Received invalid color data", G_STRFUNC);
+      return FALSE;
+    }
+
+  dropped = (guint16 *) gtk_selection_data_get_data (data);
+  rgba.red   = dropped[0] / 65535.;
+  rgba.green = dropped[1] / 65535.;
+  rgba.blue  = dropped[2] / 65535.;
+  rgba.alpha = dropped[3] / 65535.;
+
+  if (bg_colors_source_add (chooser->priv->colors_source, &rgba, &row_ref) == FALSE)
+    return FALSE;
+
+  /* Change source */
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (chooser->priv->colors_button), TRUE);
+
+  /* And select the newly added item */
+  to_focus_path = gtk_tree_row_reference_get_path (row_ref);
+  gtk_icon_view_select_path (GTK_ICON_VIEW (chooser->priv->icon_view), to_focus_path);
+  gtk_icon_view_scroll_to_path (GTK_ICON_VIEW (chooser->priv->icon_view),
+                                to_focus_path, TRUE, 1.0, 1.0);
+  gtk_tree_row_reference_free (row_ref);
+  gtk_tree_path_free (to_focus_path);
+
+  return TRUE;
+}
+
+static void
+cc_background_panel_drag_items (GtkWidget *widget,
+                                GdkDragContext *context, gint x, gint y,
+                                GtkSelectionData *data, guint info, guint time,
+                                CcBackgroundChooserDialog *chooser)
+{
+  gint i;
+  char *uri;
+  gchar **uris;
+  gboolean ret = FALSE;
+
+  if (info == COLOR)
+    {
+      ret = cc_background_panel_drag_color (chooser, data);
+      goto out;
+    }
+
+  uris = gtk_selection_data_get_uris (data);
+  if (!uris)
+    goto out;
+
+  for (i = 0; uris[i] != NULL; i++)
+    {
+      uri = uris[i];
+      if (!bg_pictures_source_is_known (chooser->priv->pictures_source, uri))
+        {
+          add_custom_wallpaper (chooser, uri);
+          ret = TRUE;
+        }
+    }
+
+  g_strfreev (uris);
+
+out:
+  gtk_drag_finish (context, ret, FALSE, time);
+}
+
 static void
 cc_background_chooser_dialog_init (CcBackgroundChooserDialog *chooser)
 {
@@ -271,6 +425,7 @@ cc_background_chooser_dialog_init (CcBackgroundChooserDialog *chooser)
   const gchar *pictures_dir;
   gchar *pictures_dir_basename;
   gchar *pictures_dir_uri;
+  GtkTargetList *target_list;
 
   chooser->priv = CC_CHOOSER_DIALOG_GET_PRIVATE (chooser);
   priv = chooser->priv;
@@ -320,6 +475,7 @@ cc_background_chooser_dialog_init (CcBackgroundChooserDialog *chooser)
   gtk_container_add (GTK_CONTAINER (hbox), button);
   g_signal_connect (button, "toggled", G_CALLBACK (on_view_toggled), chooser);
   g_object_set_data (G_OBJECT (button), "source", priv->pictures_source);
+  priv->pictures_button = button;
 
   button = gtk_radio_button_new_with_label_from_widget (GTK_RADIO_BUTTON (button1), _("Colors"));
   context = gtk_widget_get_style_context (button);
@@ -328,6 +484,7 @@ cc_background_chooser_dialog_init (CcBackgroundChooserDialog *chooser)
   gtk_container_add (GTK_CONTAINER (hbox), button);
   g_signal_connect (button, "toggled", G_CALLBACK (on_view_toggled), chooser);
   g_object_set_data (G_OBJECT (button), "source", priv->colors_source);
+  priv->colors_button = button;
 
   gtk_widget_show_all (hbox);
 
@@ -338,6 +495,16 @@ cc_background_chooser_dialog_init (CcBackgroundChooserDialog *chooser)
   gtk_widget_set_vexpand (priv->sw_content, TRUE);
   gtk_container_add (GTK_CONTAINER (grid), priv->sw_content);
 
+  /* Add drag and drop support for bg images */
+  gtk_drag_dest_set (priv->sw_content, GTK_DEST_DEFAULT_ALL, NULL, 0, GDK_ACTION_COPY);
+  target_list = gtk_target_list_new (NULL, 0);
+  gtk_target_list_add_uri_targets (target_list, URI_LIST);
+  gtk_target_list_add_table (target_list, color_targets, 1);
+  gtk_drag_dest_set_target_list (priv->sw_content, target_list);
+  gtk_target_list_unref (target_list);
+  g_signal_connect (priv->sw_content, "drag-data-received",
+                    G_CALLBACK (cc_background_panel_drag_items), chooser);
+
   priv->empty_pictures_box = gtk_grid_new ();
   gtk_widget_set_no_show_all (priv->empty_pictures_box, TRUE);
   gtk_grid_set_column_spacing (GTK_GRID (priv->empty_pictures_box), 12);
@@ -413,18 +580,12 @@ cc_background_chooser_dialog_init (CcBackgroundChooserDialog *chooser)
   g_signal_connect (priv->icon_view, "item-activated", G_CALLBACK (on_item_activated), chooser);
 
   renderer = gtk_cell_renderer_pixbuf_new ();
-  /* set stock size to 32px so that emblems render at 16px. see:
-   * https://bugzilla.gnome.org/show_bug.cgi?id=682123#c4
-   */
-  g_object_set (renderer,
-                "stock-size", GTK_ICON_SIZE_DND,
-                NULL);
   gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (priv->icon_view),
                               renderer,
                               FALSE);
   gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (priv->icon_view),
                                   renderer,
-                                  "gicon", 0,
+                                  "surface", 0,
                                   NULL);
 
   gtk_dialog_add_button (GTK_DIALOG (chooser), _("_Cancel"), GTK_RESPONSE_CANCEL);
